from pathlib import Path

import toml

from package_schemes.interfaces.base import BasePyproject
from package_schemes.schemes import PyProjectScheme


class PoetryPyproject(BasePyproject):
    def __init__(self, file: Path) -> None:
        super().__init__(file)
        data = toml.loads(file.read_text())
        self.data = PyProjectScheme.model_validate(data)



from pathlib import Path
from typing import Optional

import toml

from package_schemes.interfaces.base import BaseLock
from package_schemes.interfaces.packages import PackageMeta
from package_schemes.schemes.uv_lock import UvLockV1Scheme


class UvLockV1(BaseLock):
    def __init__(self, file: Path) -> None:
        super().__init__(file)
        data = toml.loads(file.read_text())
        self.data = UvLockV1Scheme.model_validate(data)

    def get_package_meta(self, name: str) -> Optional[PackageMeta]:
        for package in self.data.packages:
            if package.name == name:
                return PackageMeta(
                    package.name,
                    None,
                    package.version,
                    [i.name for i in package.dependencies],
                )
        return None



from collections.abc import Generator

from package_schemes.interfaces.base import BaseLock, BasePyproject
from package_schemes.interfaces.packages import Package, PackageMeta


class Project(object):
    pyproject: BasePyproject
    lock: BaseLock
    packages: dict[str, Package]

    def __init__(self, pyproject: BasePyproject, lock: BaseLock) -> None:
        self.pyproject = pyproject
        self.lock = lock
        self.packages = {}

    def get_root_packages(self) -> list[Package]:
        return [self.get_package(i) for i in self.pyproject.get_root_packages_names()]

    def get_packages(self) -> Generator[Package, None, None]:
        def get_children(parent: Package) -> Generator[Package, None, None]:
            yield parent
            for i in parent.dependencies:
                yield from get_children(i)

        for i in self.get_root_packages():
            yield from get_children(i)

    def get_package(self, name: str) -> Package:
        if name in self.packages:
            return self.packages[name]

        package_meta = PackageMeta.merge(self.pyproject.get_package_meta(name), self.lock.get_package_meta(name))

        deps = []

        for dep_package_name in package_meta.dependencies:
            deps.append(self.get_package(dep_package_name))

        self.packages[name] = Package.from_meta(package_meta, deps)

        return self.packages[name]



# type: ignore


def test_version():
    from package_schemes import __version__

    assert __version__


def test_imports():
    import package_schemes

    assert package_schemes.Package
    assert package_schemes.PoetryLock
    assert package_schemes.UvLockV1
    assert package_schemes.Project

    assert package_schemes.UvLockV1Scheme
    assert package_schemes.PyProjectScheme



from pathlib import Path
from typing import Optional

import toml

from package_schemes.interfaces.base import BasePyproject
from package_schemes.interfaces.packages import PackageMeta
from package_schemes.patterns import PACKAGE_AND_VERSION_PATTERN
from package_schemes.schemes.pyproject import PyProjectScheme


class Pyproject(BasePyproject):
    def __init__(self, file: Path) -> None:
        super().__init__(file)
        data = toml.loads(file.read_text())
        self.data = PyProjectScheme.model_validate(data)

    def get_root_packages_names(self) -> list[str]:
        packages: list[str] = []
        for dep in self.data.project.dependencies:
            match = PACKAGE_AND_VERSION_PATTERN.match(dep)
            if match:
                packages.append(match.group('name'))

        return packages

    def get_package_meta(self, name: str) -> Optional[PackageMeta]:
        for dep in self.data.project.dependencies:
            if not dep.startswith(name):
                continue

            match = PACKAGE_AND_VERSION_PATTERN.match(dep)

            if not match:
                continue

            return PackageMeta(
                match.group('name'),
                match.group('version'),
                None,
                [],
            )

        return None



from importlib import metadata as _metadata

from package_schemes.interfaces import *  # noqa: F403
from package_schemes.schemes import *  # noqa: F403

try:
    __version__ = _metadata.version('package-schemes')

except _metadata.PackageNotFoundError:
    __version__ = '0.0.0'



from __future__ import annotations

from dataclasses import dataclass, field
from typing import Optional


@dataclass()
class PackageMeta(object):
    name: str
    version_pattern: str | None
    version: str | None
    dependencies: list[str] = field(default_factory=list)

    @classmethod
    def merge(cls, meta_1: Optional[PackageMeta], meta_2: Optional[PackageMeta]):
        if meta_1 is None:
            return meta_2

        if meta_2 is None:
            return meta_1

        assert meta_1.name == meta_2.name
        name = meta_1.name

        version = None

        for i in [meta_1.version, meta_2.version]:
            if i is not None:
                version = i
                break

        assert version is not None

        version_pattern = None
        for i in [meta_1.version_pattern, meta_2.version_pattern, meta_1.version, meta_2.version]:
            if i is not None:
                version_pattern = i
                break

        assert version_pattern is not None

        deps = meta_1.dependencies + meta_2.dependencies

        return cls(
            name,
            version_pattern,
            version,
            deps,
        )


@dataclass()
class Package(object):
    name: str
    version_pattern: str | None
    version: str
    dependencies: list[Package]

    extra: str | None = None
    is_dev: bool = False

    @classmethod
    def from_meta(cls, meta: PackageMeta, deps: list[Package]):
        assert meta.version is not None

        return cls(meta.name, meta.version_pattern, meta.version, deps)
